import { p as pathBrowserify, c as cjs, g as getFS } from './props-to-nodes.esm2.js';
export { b as getRepoPath, a as propsToDocumentation } from './props-to-nodes.esm2.js';
export { NodeKind, isBlockNode, isBoldNode, isCodeNode, isCollapsibleNode, isEmphasisNode, isHeadingNode, isInlineCodeNode, isLinkNode, isNodeWithChildren, isNodeWithValue, isParagraphNode, isTableCellNode, isTableNode, isTableRowNode, isTextNode } from './types.esm.js';
import 'typescript';
import 'url';
import 'punycode';
import 'querystring';

/* eslint-disable @typescript-eslint/no-require-imports */
let importFresh;
const loadJs = function loadJs(filepath) {
    if (importFresh === undefined) {
        importFresh = require('import-fresh');
    }
    const result = importFresh(filepath);
    return result;
};
let parseJson;
const loadJson = function loadJson(filepath, content) {
    if (parseJson === undefined) {
        parseJson = require('parse-json');
    }
    try {
        const result = parseJson(content);
        return result;
    }
    catch (error) {
        error.message = `JSON Error in ${filepath}:\n${error.message}`;
        throw error;
    }
};
let yaml;
const loadYaml = function loadYaml(filepath, content) {
    if (yaml === undefined) {
        yaml = require('yaml');
    }
    try {
        const result = yaml.parse(content, { prettyErrors: true });
        return result;
    }
    catch (error) {
        error.message = `YAML Error in ${filepath}:\n${error.message}`;
        throw error;
    }
};
const loaders = { loadJs, loadJson, loadYaml };

// Resolves property names or property paths defined with period-delimited
// strings or arrays of strings. Property names that are found on the source
// object are used directly (even if they include a period).
// Nested property names that include periods, within a path, are only
// understood in array paths.
function getPropertyByPath(source, path) {
    if (typeof path === 'string' &&
        Object.prototype.hasOwnProperty.call(source, path)) {
        return source[path];
    }
    const parsedPath = typeof path === 'string' ? path.split('.') : path;
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    return parsedPath.reduce((previous, key) => {
        if (previous === undefined) {
            return previous;
        }
        return previous[key];
    }, source);
}

class ExplorerBase {
    constructor(options) {
        if (options.cache === true) {
            this.loadCache = new Map();
            this.searchCache = new Map();
        }
        this.config = options;
        this.validateConfig();
    }
    clearLoadCache() {
        if (this.loadCache) {
            this.loadCache.clear();
        }
    }
    clearSearchCache() {
        if (this.searchCache) {
            this.searchCache.clear();
        }
    }
    clearCaches() {
        this.clearLoadCache();
        this.clearSearchCache();
    }
    validateConfig() {
        const config = this.config;
        config.searchPlaces.forEach((place) => {
            const loaderKey = pathBrowserify.extname(place) || 'noExt';
            const loader = config.loaders[loaderKey];
            if (!loader) {
                throw new Error(`No loader specified for ${getExtensionDescription(place)}, so searchPlaces item "${place}" is invalid`);
            }
            if (typeof loader !== 'function') {
                throw new Error(`loader for ${getExtensionDescription(place)} is not a function (type provided: "${typeof loader}"), so searchPlaces item "${place}" is invalid`);
            }
        });
    }
    shouldSearchStopWithResult(result) {
        if (result === null)
            return false;
        if (result.isEmpty && this.config.ignoreEmptySearchPlaces)
            return false;
        return true;
    }
    nextDirectoryToSearch(currentDir, currentResult) {
        if (this.shouldSearchStopWithResult(currentResult)) {
            return null;
        }
        const nextDir = nextDirUp(currentDir);
        if (nextDir === currentDir || currentDir === this.config.stopDir) {
            return null;
        }
        return nextDir;
    }
    loadPackageProp(filepath, content) {
        const parsedContent = loaders.loadJson(filepath, content);
        const packagePropValue = getPropertyByPath(parsedContent, this.config.packageProp);
        return packagePropValue || null;
    }
    getLoaderEntryForFile(filepath) {
        if (pathBrowserify.basename(filepath) === 'package.json') {
            const loader = this.loadPackageProp.bind(this);
            return loader;
        }
        const loaderKey = pathBrowserify.extname(filepath) || 'noExt';
        const loader = this.config.loaders[loaderKey];
        if (!loader) {
            throw new Error(`No loader specified for ${getExtensionDescription(filepath)}`);
        }
        return loader;
    }
    loadedContentToCosmiconfigResult(filepath, loadedContent) {
        if (loadedContent === null) {
            return null;
        }
        if (loadedContent === undefined) {
            return { filepath, config: undefined, isEmpty: true };
        }
        return { config: loadedContent, filepath };
    }
    validateFilePath(filepath) {
        if (!filepath) {
            throw new Error('load must pass a non-empty string');
        }
    }
}
function nextDirUp(dir) {
    return pathBrowserify.dirname(dir);
}
function getExtensionDescription(filepath) {
    const ext = pathBrowserify.extname(filepath);
    return ext ? `extension "${ext}"` : 'files without extensions';
}

async function cacheWrapper(cache, key, fn) {
    const cached = cache.get(key);
    if (cached !== undefined) {
        return cached;
    }
    const result = await fn();
    cache.set(key, result);
    return result;
}

async function getDirectory(fs, filepath) {
    const filePathIsDirectory = await fs.isDirectory(filepath);
    if (filePathIsDirectory === true) {
        return filepath;
    }
    const directory = pathBrowserify.dirname(filepath);
    return directory;
}

const emptyResults = { config: {}, filepath: '' };

class Explorer extends ExplorerBase {
    constructor(options) {
        super(options);
    }
    async search(searchFrom) {
        if (!this.config.fs) {
            return emptyResults;
        }
        const startDirectory = await getDirectory(this.config.fs, searchFrom || this.config.fs.cwd());
        const result = await this.searchFromDirectory(startDirectory);
        return result;
    }
    async searchFromDirectory(dir) {
        if (!this.config.fs) {
            return emptyResults;
        }
        const absoluteDir = pathBrowserify.resolve(this.config.fs.cwd(), dir);
        const run = async () => {
            const result = await this.searchDirectory(absoluteDir);
            const nextDir = this.nextDirectoryToSearch(absoluteDir, result);
            if (nextDir) {
                return this.searchFromDirectory(nextDir);
            }
            const transformResult = await this.config.transform(result);
            return transformResult;
        };
        if (this.searchCache) {
            return cacheWrapper(this.searchCache, absoluteDir, run);
        }
        return run();
    }
    async searchDirectory(dir) {
        for await (const place of this.config.searchPlaces) {
            const placeResult = await this.loadSearchPlace(dir, place);
            if (this.shouldSearchStopWithResult(placeResult) === true) {
                return placeResult;
            }
        }
        // config not found
        return null;
    }
    async loadSearchPlace(dir, place) {
        const filepath = pathBrowserify.join(dir, place);
        if (!this.config.fs) {
            return emptyResults;
        }
        const fileContents = await this.config.fs.readFile(filepath);
        const result = await this.createCosmiconfigResult(filepath, fileContents);
        return result;
    }
    async loadFileContent(filepath, content) {
        if (content === null) {
            return null;
        }
        if (content.trim() === '') {
            return undefined;
        }
        const loader = this.getLoaderEntryForFile(filepath);
        const loaderResult = await loader(filepath, content);
        return loaderResult;
    }
    async createCosmiconfigResult(filepath, content) {
        const fileContent = await this.loadFileContent(filepath, content);
        const result = this.loadedContentToCosmiconfigResult(filepath, fileContent);
        return result;
    }
    async load(filepath) {
        if (!this.config.fs) {
            return emptyResults;
        }
        this.validateFilePath(filepath);
        const absoluteFilePath = pathBrowserify.resolve(this.config.fs.cwd(), filepath);
        const runLoad = async () => {
            if (!this.config.fs) {
                return emptyResults;
            }
            const fileContents = await this.config.fs.readFile(absoluteFilePath);
            const result = await this.createCosmiconfigResult(absoluteFilePath, fileContents);
            const transformResult = await this.config.transform(result);
            return transformResult;
        };
        if (this.loadCache) {
            return cacheWrapper(this.loadCache, absoluteFilePath, runLoad);
        }
        return runLoad();
    }
}

/**
 * original code forked from https://github.com/davidtheclark/cosmiconfig
 */
// eslint-disable-next-line @typescript-eslint/explicit-function-return-type
function cosmiconfig(moduleName, options = {}) {
    const normalizedOptions = normalizeOptions(moduleName, options);
    const explorer = new Explorer(normalizedOptions);
    return {
        search: explorer.search.bind(explorer),
        load: explorer.load.bind(explorer),
        clearLoadCache: explorer.clearLoadCache.bind(explorer),
        clearSearchCache: explorer.clearSearchCache.bind(explorer),
        clearCaches: explorer.clearCaches.bind(explorer),
    };
}
// do not allow mutation of default loaders. Make sure it is set inside options
const defaultLoaders = Object.freeze({
    '.cjs': loaders.loadJs,
    '.js': loaders.loadJs,
    '.json': loaders.loadJson,
    '.yaml': loaders.loadYaml,
    '.yml': loaders.loadYaml,
    noExt: loaders.loadYaml,
});
const identity = function identity(x) {
    return x;
};
function normalizeOptions(moduleName, options) {
    var _a;
    const defaults = {
        packageProp: moduleName,
        searchPlaces: [
            'package.json',
            `.${moduleName}rc`,
            `.${moduleName}rc.json`,
            `.${moduleName}rc.yaml`,
            `.${moduleName}rc.yml`,
            `.${moduleName}rc.js`,
            `.${moduleName}rc.cjs`,
            `${moduleName}.config.js`,
            `${moduleName}.config.cjs`,
        ],
        ignoreEmptySearchPlaces: true,
        stopDir: ((_a = options.fs) === null || _a === void 0 ? void 0 : _a.cwd()) || '',
        cache: true,
        transform: identity,
        loaders: defaultLoaders,
    };
    const normalizedOptions = {
        ...defaults,
        ...options,
        loaders: {
            ...defaults.loaders,
            ...options.loaders,
        },
    };
    return normalizedOptions;
}

/**
 * deep merge of documentation configurations, by replacing array members (ie sections, columns)
 */
const mergeConfig = (dest, src) => cjs(dest, src, {
    arrayMerge: (dest, src) => src,
});
/**
 * Read the api-docs configuration file
 * @param fileName the file that is being analyzed, will be used the starting folder to search for configuration files.
 * @param configFileName pass directly the configuration file name
 * @param options optional configuration options
 * @returns page generation options from the config file
 */
const apiDocsConfig = async (fileName, configFileName, options) => {
    const { fs, elementId, cosmic } = options || {};
    let results = emptyResults;
    if (typeof window !== 'undefined') {
        return results;
    }
    const configExplorer = cosmiconfig('api-docs', { fs: getFS(fs), ...cosmic });
    if (configFileName) {
        results = await configExplorer.load(configFileName);
    }
    else {
        const searchPath = pathBrowserify.dirname(fileName);
        results = await configExplorer.search(searchPath);
    }
    if (results) {
        const micromatch = require('micromatch');
        const { filepath, config } = results;
        if (filepath && config) {
            const { elements, ...rest } = config;
            results.config = rest;
            if (elements) {
                const id = elementId || pathBrowserify.relative(pathBrowserify.dirname(filepath), fileName);
                const onlyName = pathBrowserify.basename(fileName);
                Object.keys(elements).forEach((key) => {
                    if (onlyName === key || micromatch.isMatch(id, key)) {
                        results.config = mergeConfig(results.config, elements[key]);
                    }
                });
            }
        }
    }
    return results;
};

export { apiDocsConfig, mergeConfig };
